@@ -11,9 +11,9 @@
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
+   Lesser General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public License
+   You should have received a copy of the GNU Lesser General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
@@ -37,70 +37,74 @@ class KURL;
 /**
  * A combined button, line-edit and a popup list widget.
  *
- * @sect Detail
+ * \b Detail \n
  *
- * This widget inherits from @ref QComboBox and implements
- * the following additional functionalities:  a completion
- * object that provides both automatic and manual text
- * completion as well as text rotation features, configurable
- * key-bindings to activate these features, and a popup-menu
- * item that can be used to allow the user to set text completion
- * modes on the fly based on their preference.
+ * This widget inherits from QComboBox and implements the following
+ * additional functionalities:  a completion object that provides both automatic
+ * and manual text completion as well as text rotation features, configurable
+ * key-bindings to activate these features, and a popup-menu item that can be
+ * used to allow the user to change the text completion mode on the fly.
  *
- * To support these new features @ref KComboBox also emits a few
- * more additional signals as well.  The main ones are the
- * @ref completion( const QString& ) and @ref textRotation( KeyBindgingType )
- * signals.  The completion signal is intended to be connected to a slot
- * that will assist the user in filling out the remaining text while
- * the rotation signals is intended to be used to trasverse through all
- * possible matches whenever text completion results in multiple matches.
- * The @ref returnPressed() and @ref returnPressed( const QString& )
- * signal is emitted when the user presses the Enter/Return key.
+ * To support these new features KComboBox emits a few additional signals
+ * such as completion( const QString& ) and textRotation( KeyBindgingType ).
+ * The completion signal can be connected to a slot that will assist the user in
+ * filling out the remaining text while the rotation signal can be used to traverse
+ * through all possible matches whenever text completion results in multiple matches.
+ * Additionally, a returnPressed() and a returnPressed( const QString& )
+ * signals are emitted when the user presses the Enter/Return key.
  *
- * This widget by default creates a completion object when you invoke
- * the @ref completionObject( bool ) member function for the first time
- * or use @ref setCompletionObject( KCompletion*, bool ) to assign your
+ * KCombobox by default creates a completion object when you invoke the
+ * completionObject( bool ) member function for the first time or
+ * explicitly use setCompletionObject( KCompletion*, bool ) to assign your
  * own completion object.  Additionally, to make this widget more functional,
- * @ref KComboBox will by default handle the text rotation and completion
- * events internally whenever a completion object is created through either
- * one of the methods mentioned above.  If you do not need this functionality,
- * simply use @ref KCompletionBase::setHandleSignals( bool ) or alternatively
- * set the boolean parameter in the above methods to FALSE.
+ * KComboBox will by default handle text rotation and completion events
+ * internally whenever a completion object is created through either one of the
+ * methods mentioned above.  If you do not need this functionality, simply use
+ * KCompletionBase::setHandleSignals(bool) or alternatively set the boolean
+ * parameter in the @p setCompletionObject call to false.
  *
- * The default key-bindings for completion and rotation is determined
- * from the global settings in @ref KStdAccel.  These values, however,
- * can be overriden locally by invoking @ref KCompletionBase::setKeyBinding().
- * The values can easily be reverted back to the default setting, by simply
- * calling @ref useGlobalSettings(). An alternate method would be to default
- * individual key-bindings by usning @ref setKeyBinding() with the default
- * second argument.
+ * The default key-bindings for completion and rotation is determined from the
+ * global settings in KStdAccel. These values, however, can be overridden
+ * locally by invoking KCompletionBase::setKeyBinding(). The values can
+ * easily be reverted back to the default setting, by simply calling
+ * useGlobalSettings(). An alternate method would be to default individual
+ * key-bindings by usning setKeyBinding() with the default second argument.
  *
- * Note that if this widget is not editable ( i.e. select-only ), then only
- * one completion mode, @p CompletionAuto, will work.  All the other modes are
- * simply ignored.  The @p CompletionAuto mode in this case allows you to
- * automatically select an item from the list by trying to match the pressed
- * keycode with the first letter of the enteries in the combo box.
+ * In non-editable (select-only) mode only one completion mode, @p CompletionAuto,
+ * is available. All the other modes are simply ignored. Moreover, the CompletionAuto
+ * mode, unlike in the read-write mode, works by matching any typed key with the
+ * first letter of entries in the combobox.
  *
- * @sect Useage
+ * A read-only KComboBox will have the same background color as a 
+ * disabled KComboBox, but its foreground color will be the one used for
+ * the read-write mode. This differs from QComboBox's implementation
+ * and is done to give visual distinction between the three different modes: 
+ * disabled, read-only, and read-write.
+ * 
+ * \b Usage \n
  *
  * To enable the basic completion feature:
  *
- * <pre>
+ * \code
  * KComboBox *combo = new KComboBox( true, this, "mywidget" );
  * KCompletion *comp = combo->completionObject();
  * // Connect to the return pressed signal - optional
- * connect(combo,SIGNAL(returnPressed(const QString&)),comp,SLOT(addItem(const QString&));
- * </pre>
+ * connect(combo,SIGNAL(returnPressed(const QString&)),comp,SLOT(addItem(const QString&)));
+ *
+ * // Provide the to be completed strings. Note that those are separate from the combo's
+ * // contents.
+ * comp->insertItems( someQStringList );
+ * \endcode
  *
  * To use your own completion object:
  *
- * <pre>
+ * \code
  * KComboBox *combo = new KComboBox( this,"mywidget" );
  * KURLCompletion *comp = new KURLCompletion();
  * combo->setCompletionObject( comp );
  * // Connect to the return pressed signal - optional
- * connect(combo,SIGNAL(returnPressed(const QString&)),comp,SLOT(addItem(const QString&));
- * </pre>
+ * connect(combo,SIGNAL(returnPressed(const QString&)),comp,SLOT(addItem(const QString&)));
+ * \endcode
  *
  * Note that you have to either delete the allocated completion object
  * when you don't need it anymore, or call
@@ -108,18 +112,14 @@ class KURL;
  *
  * Miscellaneous function calls:
  *
- * <pre>
+ * \code
  * // Tell the widget not to handle completion and rotation
  * combo->setHandleSignals( false );
  * // Set your own completion key for manual completions.
  * combo->setKeyBinding( KCompletionBase::TextCompletion, Qt::End );
  * // Hide the context (popup) menu
  * combo->setContextMenuEnabled( false );
- * // Temporarly disable signal emition
- * combo->disableSignals();
- * // Default the all key-bindings to their system-wide settings.
- * combo->useGlobalKeyBindings();
- * </pre>
+ * \endcode
  *
  * @short An enhanced combo box.
  * @author Dawit Alemayehu <adawit@kde.org>
@@ -134,7 +134,7 @@ class KComboBox : public QComboBox, public KCompletionBase
 public:
 
     /**
-    * Construct a read-only or rather select-only combo box with a
+    * Constructs a read-only or rather select-only combo box with a
     * parent object and a name.
     *
     * @param parent The parent object of this widget
@@ -143,7 +143,7 @@ public:
     KComboBox( QWidget *parent=0, const char *name=0 );
 
     /**
-    * Construct a "read-write" or "read-only" combo box depending on
+    * Constructs a "read-write" or "read-only" combo box depending on
     * the value of the first argument( @p rw ) with a parent, a
     * name.
     *
@@ -160,14 +160,14 @@ public:
 
     /**
      * Sets @p url into the edit field of the combobox. It uses
-     * @ref KURL::prettyURL() so that the url is properly decoded for
+     * KURL::prettyURL() so that the url is properly decoded for
      * displaying.
      */
     void setEditURL( const KURL& url );
 
     /**
      * Inserts @p url at position @p index into the combobox. The item will
-     * be appended if @p index is negative. @ref KURL::prettyURL() is used
+     * be appended if @p index is negative. KURL::prettyURL() is used
      * so that the url is properly decoded for displaying.
      */
     void insertURL( const KURL& url, int index = -1 );
@@ -175,27 +175,27 @@ public:
     /**
      * Inserts @p url with the pixmap &p pixmap at position @p index into
      * the combobox. The item will be appended if @p index is negative.
-     * @ref KURL::prettyURL() is used so that the url is properly decoded
+     * KURL::prettyURL() is used so that the url is properly decoded
      * for displaying.
      */
     void insertURL( const QPixmap& pixmap, const KURL& url, int index = -1 );
 
     /**
      * Replaces the item at position @p index with @p url.
-     * @ref KURL::prettyURL() is used so that the url is properly decoded
+     * KURL::prettyURL() is used so that the url is properly decoded
      * for displaying.
      */
     void changeURL( const KURL& url, int index );
 
     /**
      * Replaces the item at position @p index with @p url and pixmap @p pixmap.
-     * @ref KURL::prettyURL() is used so that the url is properly decoded
+     * KURL::prettyURL() is used so that the url is properly decoded
      * for displaying.
      */
     void changeURL( const QPixmap& pixmap, const KURL& url, int index );
 
     /**
-    * Retreive the current cursor position.
+    * Returns the current cursor position.
     *
     * This method always returns a -1 if the combo-box is @em not
     * editable (read-write).
@@ -205,12 +205,12 @@ public:
     int cursorPosition() const { return ( lineEdit() ) ? lineEdit()->cursorPosition() : -1; }
 
     /**
-    * Re-implemented from @ref QComboBox.
+    * Re-implemented from QComboBox.
     *
     * If @p true, the completion mode will be set to automatic.
-    * Otherwise, it is defaulted to the gloabl setting.  This
-    * methods has been replaced by the more comprehensive
-    * @ref setCompletionMode().
+    * Otherwise, it is defaulted to the global setting.  This
+    * method has been replaced by the more comprehensive
+    * setCompletionMode().
     *
     * @param autocomplete Flag to enable/disable automatic completion mode.
     */
@@ -221,41 +221,40 @@ public:
     *
     * Returns @p true if the current completion mode is set
     * to automatic.  See its more comprehensive replacement
-    * @ref completionMode().
+    * completionMode().
     *
     * @return @p true when completion mode is automatic.
     */
-    bool autoCompletion() const { return completionMode() == KGlobalSettings::CompletionAuto; }
+    bool autoCompletion() const {
+        return completionMode() == KGlobalSettings::CompletionAuto;
+    }
 
     /**
-    * Enable or disable the popup (context) menu.
+    * Enables or disable the popup (context) menu.
     *
     * This method only works if this widget is editable, i.e.
     * read-write and allows you to enable/disable the context
     * menu.  It does nothing if invoked for a none-editable
     * combo-box.  Note that by default the mode changer item
     * is made visiable whenever the context menu is enabled.
-    * Use @ref hideModechanger() if you want to hide this
+    * Use hideModechanger() if you want to hide this
     * item.    Also by default, the context menu is created if
     * this widget is editable. Call this function with the
     * argument set to false to disable the popup menu.
     *
     * @param showMenu If @p true, show the context menu.
-    * @param showMode If @p true, show the mode changer.
     */
     virtual void setContextMenuEnabled( bool showMenu );
 
     /**
     * Returns @p true when the context menu is enabled.
-    *
-    * @return @p true if context menu is enabled.
     */
     bool isContextMenuEnabled() const { return m_bEnableMenu; }
 
     /**
      * Enables/Disables handling of URL drops. If enabled and the user
      * drops an URL, the decoded URL will be inserted. Otherwise the default
-     * behaviour of QComboBox is used, which inserts the encoded URL.
+     * behavior of QComboBox is used, which inserts the encoded URL.
      *
      * @param enable If @p true, insert decoded URLs
      */
@@ -263,18 +262,9 @@ public:
 
     /**
      * Returns @p true when decoded URL drops are enabled
-     *
-     * @return @p true if decoded URL drops are enabled
      */
     bool isURLDropsEnabled() const;
 
-    /**
-    * Returns @p true if the combo-box is editable.
-    *
-    * @return @p true if combo is editable.
-    */
-    bool isEditable() const { return editable(); }
-
     /**
      * Convenience method which iterates over all items and checks if
      * any of them is equal to @p text.
@@ -287,12 +277,12 @@ public:
     bool contains( const QString& text ) const;
 
     /**
-     * By default, @ref KComboBox recognizes Key_Return and Key_Enter
+     * By default, KComboBox recognizes Key_Return and Key_Enter
      * and emits
-     * the @ref returnPressed() signals, but it also lets the event pass,
+     * the returnPressed() signals, but it also lets the event pass,
      * for example causing a dialog's default-button to be called.
      *
-     * Call this method with @p trap equal to @p true to make @ref KComboBox
+     * Call this method with @p trap equal to true to make KComboBox
      * stop these
      * events. The signals will still be emitted of course.
      *
@@ -312,78 +302,69 @@ public:
 
     /**
     * Re-implemented for internal reasons.  API not affected.
-    *
-    * @reimplemented
     */
     virtual bool eventFilter( QObject *, QEvent * );
 
-    /**
-    * Re-implemented from @ref KCompletionBase for internal reasons.
-    */
-    virtual void setCompletionMode( KGlobalSettings::Completion mode );
-
     /**
      * @returns the completion-box, that is used in completion mode
-     * @ref KGlobalSettings::CompletionPopup.
+     * KGlobalSettings::CompletionPopup and KGlobalSettings::CompletionPopupAuto.
      * This method will create a completion-box by calling
-     * @ref makeCompletionBox, if none is there, yet.
+     * KLineEdit::completionBox, if none is there, yet.
+     *
+     * @param create Set this to false if you don't want the box to be created
+     *               i.e. to test if it is available.
      */
-    KCompletionBox * completionBox();
+    KCompletionBox * completionBox( bool create = true );
 
     /**
-     * @returns the completion box or 0L if none is available
+     * Re-implemented for internal reasons.  API remains unaffected.
+     * NOTE: Only editable comboboxes can have a line editor. As such
+     * any attempt to assign a line-edit to a non-editable combobox will
+     * simply be ignored.
      */
-    KCompletionBox * completionBox( bool create );
-
-    /**
-     * Reimplemented for internal reasons, the API is not affected.
-     */
-    virtual void setCompletionObject( KCompletion *, bool hsig = true );
+    virtual void setLineEdit( QLineEdit * );
 
 signals:
     /**
     * Emitted when the user presses the Enter key.
     *
-    * Note that this signal is only
-    * emitted if this widget is editable.
+    * Note that this signal is only emitted when the widget is editable.
     */
     void returnPressed();
 
     /**
-    * Emitted when the user presses
-    * the Enter key.
+    * Emitted when the user presses the Enter key.
     *
-    * The argument is the current
-    * text being edited.  This signal is just like
-    * @ref returnPressed() except it contains the
-    * current text as its argument.
+    * The argument is the current text being edited.  This signal is just like
+    * returnPressed() except it contains the current text as its argument.
     *
-    * Note that this signal is only emitted if this
+    * Note that this signal is only emitted when the
     * widget is editable.
     */
     void returnPressed( const QString& );
 
     /**
-    * This signal is emitted when the completion key
-    * is pressed.
+    * Emitted when the completion key is pressed.
     *
-    * The argument is the current text
-    * being edited.
+    * The argument is the current text being edited.
     *
-    * Note that this signal is @em not available if this
-    * widget is non-editable or the completion mode is
-    * set to @p KGlobalSettings::CompletionNone.
+    * Note that this signal is @em not available when the widget is non-editable
+    * or the completion mode is set to @p KGlobalSettings::CompletionNone.
     */
     void completion( const QString& );
 
     /**
+     * Emitted when the shortcut for substring completion is pressed.
+     */
+    void substringCompletion( const QString& );
+
+   /**
     * Emitted when the text rotation key-bindings are pressed.
     *
-    * The argument indicates which key-binding was pressed.
-    * In this case this can be either one of four values:
-    * @p PrevCompletionMatch, @p NextCompletionMatch, @p RotateUp or
-    * @p RotateDown. See @ref KCompletionBase::setKeyBinding() for
-    * details.
+    * The argument indicates which key-binding was pressed. In this case this
+    * can be either one of four values: @p PrevCompletionMatch,
+    * @p NextCompletionMatch, @p RotateUp or @p RotateDown. See
+    * KCompletionBase::setKeyBinding() for details.
     *
     * Note that this signal is @em NOT emitted if the completion
     * mode is set to CompletionNone.
@@ -391,29 +372,26 @@ signals:
     void textRotation( KCompletionBase::KeyBindingType );
 
     /**
-     * Emitted when the user changed the completion mode by using the
-     * popupmenu.
+     * Emitted whenever the completion mode is changed by the user
+     * through the context menu.
      */
     void completionModeChanged( KGlobalSettings::Completion );
 
     /**
      * Emitted before the context menu is displayed.
      *
-     * The signal allows you to add your own entries into the
-     * the context menu that is created on demand.
+     * The signal allows you to add your own entries into the context menu.
+     * Note that you MUST NOT store the pointer to the QPopupMenu since it is
+     * created and deleted on demand.  Otherwise, you can crash your app.
      *
-     * NOTE: Do not store the pointer to the QPopupMenu
-     * provided through since it is created and deleted
-     * on demand.
-     *
-     * @param the context menu about to be displayed
+     * @param p the context menu about to be displayed
      */
-    void aboutToShowContextMenu( QPopupMenu * );
+    void aboutToShowContextMenu( QPopupMenu * p );
 
 public slots:
 
     /**
-    * Iterate through all possible matches of the completed text
+    * Iterates through all possible matches of the completed text
     * or the history list.
     *
     * Depending on the value of the argument, this function either
@@ -429,26 +407,38 @@ public slots:
     *
     * @param type The key-binding invoked.
     */
-    void rotateText( KCompletionBase::KeyBindingType /* type */ );
+    void rotateText( KCompletionBase::KeyBindingType type );
 
     /**
      * Sets the completed text in the line-edit appropriately.
      *
      * This function is an implementation for
-     * @ref KCompletionBase::setCompletedText.
+     * KCompletionBase::setCompletedText.
      */
     virtual void setCompletedText( const QString& );
 
     /**
-     * Sets @p items into the completion-box if @ref completionMode() is
+     * Sets @p items into the completion-box if completionMode() is
      * CompletionPopup. The popup will be shown immediately.
      */
     void setCompletedItems( const QStringList& items );
 
+    /**
+     * Selects the first item that matches @p item. If there is no such item,
+     * it is inserted at position @p index if @p insert is true. Otherwise,
+     * no item is selected.
+     */
+    void setCurrentItem( const QString& item, bool insert = false, int index = -1 );
+    /**
+     * Simply calls QComboBox' implementation. Only here to not become
+     * shadowed.
+     */
+    void setCurrentItem(int index) { QComboBox::setCurrentItem(index); }
+
 protected slots:
 
     /**
-    * @deprecated.
+    * @deprecated
     */
     virtual void itemSelected( QListBoxItem* ) {};
 
@@ -465,24 +455,7 @@ protected slots:
     */
     virtual void makeCompletion( const QString& );
 
-    /**
-     * @deprecated
-     */
-    // BCI: Remove me in KDE 3.0
-    void slotAboutToShow() {}
-
-    /**
-     * @deprecated
-     */
-    // BCI: Remove me in KDE 3.0
-    void slotCancelled() {}
-
 protected:
-    /**
-    * @reimplemented
-    */
-    virtual void keyPressEvent ( QKeyEvent* );
-
     /*
     * This function simply sets the lineedit text and
     * highlights the text appropriately if the boolean
@@ -499,49 +472,23 @@ protected:
     virtual void create( WId = 0, bool initializeWindow = true,
                          bool destroyOldWindow = true );
 
+    virtual void wheelEvent( QWheelEvent *ev );
+
+private slots:
+    void lineEditDeleted();
+
 private:
-    // Constants that represent the ID's of the popup menu.
-    // TODO: See if we can replace this mess with KActionMenu
-    // in the future though this is working lovely.
-    enum MenuID {
-        Default=0,
-        Cut,
-        Copy,
-        Paste,
-        Clear,
-        Unselect,
-        SelectAll,
-        NoCompletion,
-        AutoCompletion,
-        ShellCompletion,
-        PopupCompletion,
-        SemiAutoCompletion
-    };
-
-    /** Initializes the variables upon construction. */
-    void init();
-    /** Creates the completion box */
-    void makeCompletionBox();
-    /** Returns the context menu, creates a new one if did not exist. */
-    QPopupMenu* contextMenuInternal();
-    /** initializes the context menu */
-    void initPopup();
-
     /**
-     * Temporary functions to delete words back and foward until
-     * alternatives are available in QT3 (Seth Chaiklin, 21 may 2001)
+     * Initializes the variables upon construction.
      */
-    void deleteWordBack();
-    void deleteWordForward();
+    void init();
+    bool m_bEnableMenu; // ### BCI: unused, remove in KDE4
+    bool m_trapReturnKey; // ### BCI: unused, remove in KDE4
 
-    bool m_bEnableMenu;
-
-    // Pointer to the line editor.
-    QGuardedPtr<QLineEdit> m_pEdit; // ### remove, don't use, use lineEdit()!
-
-    // indicating if we should stop return-key events from propagating
-    bool m_trapReturnKey;
+protected:
+    virtual void virtual_hook( int id, void* data );
 
+private:
     class KComboBoxPrivate;
     KComboBoxPrivate *d;
 };
@@ -556,6 +503,12 @@ class KPixmapProvider;
  * load and save the completion list to preserve the weighting between
  * sessions.
  *
+ * KHistoryCombo obeys the HISTCONTROL environment variable to determine
+ * whether duplicates in the history should be tolerated in
+ * addToHistory() or not. During construction of KHistoryCombo,
+ * duplicates will be disabled when HISTCONTROL is set to "ignoredups" or
+ * "ignoreboth". Otherwise, duplicates are enabled by default.
+ *
  * @author Carsten Pfeiffer <pfeiffer@kde.org>
  * @short A combobox for offering a history and completion
  */
@@ -571,15 +524,15 @@ public:
      * Completion will be used automatically for the items in the combo.
      *
      * The insertion-policy is set to NoInsertion, you have to add the items
-     * yourself via the slot @ref addToHistory. If you want every item added,
+     * yourself via the slot addToHistory. If you want every item added,
      * use
      *
-     * <pre>
+     * \code
      * connect( combo, SIGNAL( activated( const QString& )),
      *          combo, SLOT( addToHistory( const QString& )));
-     * </pre>
+     * \endcode
      *
-     * Use @ref QComboBox::setMaxCount() to limit the history.
+     * Use QComboBox::setMaxCount() to limit the history.
      *
      * @p parent the parent object of this widget.
      * @p name the name of this widget.
@@ -603,9 +556,9 @@ public:
 
     /**
      * Inserts @p items into the combobox. @p items might get
-     * truncated if it is longer than @ref maxCount()
+     * truncated if it is longer than maxCount()
      *
-     * @see #historyItems
+     * @see historyItems
      */
     inline void setHistoryItems( QStringList items ) {
         setHistoryItems(items, false);
@@ -613,14 +566,14 @@ public:
 
     /**
      * Inserts @p items into the combobox. @p items might get
-     * truncated if it is longer than @ref maxCount()
+     * truncated if it is longer than maxCount()
      *
      * Set @p setCompletionList to true, if you don't have a list of
      * completions. This tells KHistoryCombo to use all the items for the
      * completion object as well.
      * You won't have the benefit of weighted completion though, so normally
      * you should do something like
-     * <pre>
+     * \code
      * KConfig *config = kapp->config();
      * QStringList list;
      *
@@ -638,7 +591,7 @@ public:
      * config->writeEntry( "Completion list", list );
      * list = combo->historyItems();
      * config->writeEntry( "History list", list );
-     * </pre>
+     * \endcode
      *
      * Be sure to use different names for saving with KConfig if you have more
      * than one KHistoryCombo.
@@ -647,7 +600,7 @@ public:
      * KCompletion object with mode KCompletion::Insertion and the mode is set
      * to KCompletion::Weighted afterwards.
      *
-     * @see #historyItems
+     * @see historyItems
      * @see KComboBox::completionObject
      * @see KCompletion::setItems
      * @see KCompletion::items
@@ -658,56 +611,56 @@ public:
      * Returns the list of history items. Empty, when this is not a read-write
      * combobox.
      *
-     * @see #setHistoryItems
+     * @see setHistoryItems
      */
     QStringList historyItems() const;
 
     /**
-     * Remove all items named @p item.
+     * Removes all items named @p item.
      *
      * @return @p true if at least one item was removed.
      *
-     * @see #addToHistory
+     * @see addToHistory
      */
     bool removeFromHistory( const QString& item );
 
     /**
      * Sets a pixmap provider, so that items in the combobox can have a pixmap.
-     * @ref KPixmapProvider is just an abstract class with the one pure virtual
-     * method @ref KPixmapProvider::pixmapFor(). This method is called whenever
+     * KPixmapProvider is just an abstract class with the one pure virtual
+     * method KPixmapProvider::pixmapFor(). This method is called whenever
      * an item is added to the KHistoryComboBox. Implement it to return your
-     * own custom pixmaps, or use the @ref KURLPixmapProvider from libkio,
-     * which uses @ref KMimeType::pixmapForURL to resolve icons.
+     * own custom pixmaps, or use the KURLPixmapProvider from libkio,
+     * which uses KMimeType::pixmapForURL to resolve icons.
      *
      * Set @p prov to 0L if you want to disable pixmaps. Default no pixmaps.
      *
-     * @see #pixmapProvider
+     * @see pixmapProvider
      */
     void setPixmapProvider( KPixmapProvider *prov );
 
     /**
      * @returns the current pixmap provider.
-     * @see #setPixmapProvider
+     * @see setPixmapProvider
      * @see KPixmapProvider
      */
     KPixmapProvider * pixmapProvider() const { return myPixProvider; }
 
     /**
      * Resets the current position of the up/down history. Call this
-     * when you manually call @ref setCurrentItem() or @ref clearEdit().
+     * when you manually call setCurrentItem() or clearEdit().
      */
     void reset() { slotReset(); }
 
 public slots:
     /**
      * Adds an item to the end of the history list and to the completion list.
-     * If @ref maxCount() is reached, the first item of the list will be
+     * If maxCount() is reached, the first item of the list will be
      * removed.
      *
      * If the last inserted item is the same as @p item, it will not be
      * inserted again.
      *
-     * If @ref duplicatesEnabled() is false, any equal existing item will be
+     * If duplicatesEnabled() is false, any equal existing item will be
      * removed before @p item is added.
      *
      * Note: By using this method and not the Q and KComboBox insertItem()
@@ -715,7 +668,7 @@ public slots:
      * completion. It would be annoying if completion would give an item
      * not in the combobox, and vice versa.
      *
-     * @see #removeFromHistory
+     * @see removeFromHistory
      * @see QComboBox::setDuplicatesEnabled
      */
     void addToHistory( const QString& item );
@@ -738,28 +691,33 @@ protected:
     virtual void keyPressEvent( QKeyEvent * );
 
     /**
-     * Inserts @p items into the combo, honouring @ref pixmapProvider()
+     * Handling wheel-events, to rotate the items.
+     */
+    virtual void wheelEvent( QWheelEvent *ev );
+
+    /**
+     * Inserts @p items into the combo, honoring pixmapProvider()
      * Does not update the completionObject.
      *
-     * Note: @ref duplicatesEnabled() is not honored here.
+     * Note: duplicatesEnabled() is not honored here.
      *
-     * Called from @ref setHistoryItems() and @ref setPixmapProvider()
+     * Called from setHistoryItems() and setPixmapProvider()
      */
     void insertItems( const QStringList& items );
 
     /**
      * @returns if we can modify the completion object or not.
      */
-    bool useCompletion() { return completionObject(false, true) != 0L; }
+    bool useCompletion() const { return compObj() != 0L; }
 
 private slots:
     /**
-     * resets the iterate index to -1
+     * Resets the iterate index to -1
      */
     void slotReset();
 
     /**
-     * called from the popupmenu
+     * Called from the popupmenu,
      * calls clearHistory() and emits cleared()
      */
     void slotClear();
@@ -771,9 +729,11 @@ private slots:
 
 private:
     void init( bool useCompletion );
+    void rotateUp();
+    void rotateDown();
 
     /**
-     * the current position (index) in the combobox, used for Up and Down
+     * The current position (index) in the combobox, used for Up and Down
      */
     int myIterateIndex;
 
@@ -789,6 +749,8 @@ private:
     bool myRotated;
     KPixmapProvider *myPixProvider;
 
+protected:
+    virtual void virtual_hook( int id, void* data );
 private:
     class KHistoryComboPrivate;
     KHistoryComboPrivate *d;
@@ -796,3 +758,4 @@ private:
 
 
 #endif
+
