@@ -3,6 +3,7 @@
  *
  *  Copyright (c) 1997 Patrick Dowler <dowler@morgul.fsh.uvic.ca>
  *  Copyright (c) 2000 Dirk A. Mueller <mueller@kde.org>
+ *  Copyright (c) 2002 Marc Mutz <mutz@kde.org>
  *
  *  Requires the Qt widget libraries, available at no cost at
  *  http://www.troll.no/
@@ -28,11 +29,13 @@
 
 #include <qwidget.h>
 #include <qspinbox.h>
+#include <kdemacros.h>
 
 class QLabel;
 class QSlider;
 class QLineEdit;
 class QLayout;
+class QValidator;
 
 class KIntSpinBox;
 
@@ -43,10 +46,10 @@ class KIntSpinBox;
  * for a different variable type
  *
  */
-
 class KNumInput : public QWidget
 {
     Q_OBJECT
+    Q_PROPERTY( QString label READ label WRITE setLabel )
 public:
     /**
      * Default constructor
@@ -55,17 +58,17 @@ public:
     KNumInput(QWidget* parent=0, const char* name=0);
 
     /**
-     * @param below A pointer to another @ref KNumInput.
+     * @param below A pointer to another KNumInput.
      *
      */
     KNumInput(KNumInput* below, QWidget* parent=0, const char* name=0);
     ~KNumInput();
 
     /**
-     * Set the text and alignment of the main description label.
+     * Sets the text and alignment of the main description label.
      *
      * @param label The text of the label.
-     *              Use @ref QString::null to remove an existing one.
+     *              Use QString::null to remove an existing one.
      *
      * @param a one of @p AlignLeft, @p AlignHCenter, YAlignRight and
      *          @p AlignTop, @p AlignVCenter, @p AlignBottom.
@@ -79,10 +82,21 @@ public:
      *     @li @p AlignBottom  The label is placed below the edit/slider
      *
      */
-    virtual void setLabel(QString label, int a = AlignLeft | AlignTop);
+    virtual void setLabel(const QString & label, int a = AlignLeft | AlignTop);
 
     /**
-     * Set the spacing of tickmarks for the slider.
+     * @return the text of the label.
+     */
+    QString label() const;
+
+    /**
+     * @return if the num input has a slider.
+     * @since 3.1
+     */
+    bool showSlider() const { return m_slider; }
+
+    /**
+     * Sets the spacing of tickmarks for the slider.
      *
      * @param minor Minor tickmark separation.
      * @param major Major tickmark separation.
@@ -90,13 +104,13 @@ public:
     void setSteps(int minor, int major);
 
     /**
-     * Specify that this widget may stretch horizontally, but is
-     * fixed vertically (like @ref QSpinBox itself).
+     * Specifies that this widget may stretch horizontally, but is
+     * fixed vertically (like QSpinBox itself).
      */
     QSizePolicy sizePolicy() const;
 
     /**
-     * Return a size which fits the contents of the control.
+     * Returns a size which fits the contents of the control.
      *
      * @return the preferred size necessary to show the control
      */
@@ -105,16 +119,16 @@ public:
 protected:
     /**
      * Call this function whenever you change something in the geometry
-     * of your @ref KNumInput child.
+     * of your KNumInput child.
      *
      */
     void layout(bool deep);
 
     /**
-     * You need to overwrite this method and implement yourlayout
-     * calculations there. 
+     * You need to overwrite this method and implement your layout
+     * calculations there.
      *
-     * See @ref KIntNumInput / @ref KDoubleNumInput implementation
+     * See KIntNumInput::doLayout and KDoubleNumInput::doLayout implementation
      * for details.
      *
      */
@@ -132,6 +146,9 @@ protected:
 private:
     void init();
 
+protected:
+    virtual void virtual_hook( int id, void* data );
+private:
     class KNumInputPrivate;
     KNumInputPrivate *d;
 };
@@ -139,7 +156,7 @@ private:
 /* ------------------------------------------------------------------------ */
 
 /**
- * KIntNumInput combines a @ref QSpinBox and optionally a @ref QSlider
+ * KIntNumInput combines a QSpinBox and optionally a QSlider
  * with a label to make an easy to use control for setting some integer
  * parameter. This is especially nice for configuration dialogs,
  * which can have many such combinated controls.
@@ -154,18 +171,23 @@ private:
  * QSpinBox (the remaining portion is used by the slider). This makes
  * it very simple to have all the sliders in a column be the same size.
  *
- * It uses @ref KIntValidator validator class. KIntNumInput enforces the
+ * It uses KIntValidator validator class. KIntNumInput enforces the
  * value to be in the given range, and can display it in any base
  * between 2 and 36.
  *
+ * \image html kintnuminput.png "KDE Int Number Input Spinbox"
+ *
  * @short An input widget for integer numbers, consisting of a spinbox and a slider.
- * @version $Id: knuminput.h,v 1.29 2001/07/28 22:30:19 mueller Exp $
+ * @version $Id: knuminput.h,v 1.56 2003/10/07 22:53:26 mueller Exp $
  */
 
 class KIntNumInput : public KNumInput
 {
     Q_OBJECT
     Q_PROPERTY( int value READ value WRITE setValue )
+    Q_PROPERTY( int minValue READ minValue WRITE setMinValue )
+    Q_PROPERTY( int maxValue READ maxValue WRITE setMaxValue )
+    Q_PROPERTY( int referencePoint READ referencePoint WRITE setReferencePoint )
     Q_PROPERTY( QString suffix READ suffix WRITE setSuffix )
     Q_PROPERTY( QString prefix READ prefix WRITE setPrefix )
     Q_PROPERTY( QString specialValueText READ specialValueText WRITE setSpecialValueText )
@@ -198,7 +220,7 @@ public:
      * Note that these two KNumInput's need not to have the same parent widget
      * or be in the same layout group.
      * The effect is that it'll adjust it's layout in correspondence
-     * with the layout of the other KNumInput's (you can build an arbitary long
+     * with the layout of the other KNumInput's (you can build an arbitrary long
      * chain).
      *
      * @param below  append KIntNumInput to the KNumInput chain
@@ -220,27 +242,58 @@ public:
      * @return the current value.
      */
     int value() const;
+
+    /**
+     * @return the curent value in units of the referencePoint.
+     * @since 3.1
+     */
+    double relativeValue() const;
+
+    /**
+     * @return the current reference point
+     * @since 3.1
+     */
+    int referencePoint() const;
+
     /**
      * @return the suffix displayed behind the value.
-     * @see #setSuffix()
+     * @see setSuffix()
      */
     QString suffix() const;
     /**
      * @return the prefix displayed in front of the value.
-     * @see #setPrefix()
+     * @see setPrefix()
      */
     QString prefix() const;
     /**
      * @return the string displayed for a special value.
-     * @see #setSpecialValueText()
+     * @see setSpecialValueText()
      */
     QString specialValueText() const;
+
     /**
-     * @param lower  lower bound on range
-     * @param upper  upper bound on range
-     * @param step   step size for the QSlider
+     * @param min  minimum value
+     * @param max  maximum value
+     * @param step step size for the QSlider
+     * @param slider whether the slider is created or not
      */
-    void setRange(int lower, int upper, int step=1, bool slider=true);
+    void setRange(int min, int max, int step=1, bool slider=true);
+    /**
+     * Sets the minimum value.
+     */
+    void setMinValue(int min);
+    /**
+     * @return the minimum value.
+     */
+    int minValue() const;
+    /**
+     * Sets the maximum value.
+     */
+    void setMaxValue(int max);
+    /**
+     * @return the maximum value.
+     */
+    int maxValue() const;
 
     /**
      * Sets the special value text. If set, the SpinBox will display
@@ -250,10 +303,7 @@ public:
      */
     void setSpecialValueText(const QString& text);
 
-    /**
-     * @reimplemented
-     */
-    virtual void setLabel(QString label, int a = AlignLeft | AlignTop);
+    virtual void setLabel(const QString & label, int a = AlignLeft | AlignTop);
 
     /**
      * This method returns the minimum size necessary to display the
@@ -270,6 +320,18 @@ public slots:
      */
     void setValue(int);
 
+    /**
+     * Sets the value in units of the referencePoint
+     * @since 3.1
+     */
+    void setRelativeValue(double);
+
+    /**
+     * Sets the reference point for relativeValue.
+     * @since 3.1
+     */
+    void setReferencePoint(int);
+
     /**
      * Sets the suffix to @p suffix.
      * Use QString::null to disable this feature.
@@ -298,22 +360,24 @@ public slots:
 
 signals:
     /**
-     * Emitted every time the value changes (by calling @ref setValue() or
+     * Emitted every time the value changes (by calling setValue() or
      * by user interaction).
      */
     void valueChanged(int);
 
+    /**
+     * Emitted whenever valueChanged is. Contains the change
+     * relative to the referencePoint.
+     * @since 3.1
+     */
+    void relativeValueChanged(double);
+
 private slots:
     void spinValueChanged(int);
+    void slotEmitRelativeValueChanged(int);
 
 protected:
-    /**
-     * @reimplemented
-     */
     virtual void doLayout();
-    /**
-     * @reimplemented
-     */
     void resizeEvent ( QResizeEvent * );
 
     KIntSpinBox* m_spin;
@@ -322,6 +386,9 @@ protected:
 private:
     void init(int value, int _base);
 
+protected:
+    virtual void virtual_hook( int id, void* data );
+private:
     class KIntNumInputPrivate;
     KIntNumInputPrivate *d;
 };
@@ -332,7 +399,7 @@ private:
 class KDoubleLine;
 
 /**
- * KDoubleNumInput combines a @ref QSpinBox and optionally a @ref QSlider
+ * KDoubleNumInput combines a QSpinBox and optionally a QSlider
  * with a label to make an easy to use control for setting some float
  * parameter. This is especially nice for configuration dialogs,
  * which can have many such combinated controls.
@@ -340,17 +407,20 @@ class KDoubleLine;
  * The slider is created only when the user specifies a range
  * for the control using the setRange function with the slider
  * parameter set to "true".
- *     
+ *
  * A special feature of KDoubleNumInput, designed specifically for
- * the situation when there are several KDoubleNumInputs in a column,
+ * the situation when there are several instances in a column,
  * is that you can specify what portion of the control is taken by the
  * QSpinBox (the remaining portion is used by the slider). This makes
  * it very simple to have all the sliders in a column be the same size.
  *
- * It uses KFloatValidator validator class. KDoubleNumInput enforces the
- * value to be in the given range.
+ * It uses the KDoubleValidator validator class. KDoubleNumInput
+ * enforces the value to be in the given range, but see the class
+ * documentation of KDoubleSpinBox for the tricky
+ * interrelationship of precision and values. All of what is said
+ * there applies here, too.
  *
- * @see KIntNumInput
+ * @see KIntNumInput, KDoubleSpinBox
  * @short An input control for real numbers, consisting of a spinbox and a slider.
  */
 
@@ -358,24 +428,44 @@ class KDoubleNumInput : public KNumInput
 {
     Q_OBJECT
     Q_PROPERTY( double value READ value WRITE setValue )
+    Q_PROPERTY( double minValue READ minValue WRITE setMinValue )
+    Q_PROPERTY( double maxValue READ maxValue WRITE setMaxValue )
     Q_PROPERTY( QString suffix READ suffix WRITE setSuffix )
     Q_PROPERTY( QString prefix READ prefix WRITE setPrefix )
     Q_PROPERTY( QString specialValueText READ specialValueText WRITE setSpecialValueText )
+    Q_PROPERTY( int precision READ precision WRITE setPrecision )
 
 public:
     /**
      * Constructs an input control for double values
-     * with initial value 0.0.
+     * with initial value 0.00.
      */
     KDoubleNumInput(QWidget *parent=0, const char *name=0);
+
     /**
+     * @deprecated (value is rounded to a multiple of 1/100)
      * Constructor
      *
      * @param value  initial value for the control
      * @param parent parent QWidget
      * @param name   internal name for this widget
      */
-    KDoubleNumInput(double value, QWidget *parent=0, const char *name=0);
+    KDoubleNumInput(double value, QWidget *parent=0, const char *name=0) KDE_DEPRECATED;
+
+    /**
+     * Constructor
+     *
+     * @param lower lower boundary value
+     * @param upper upper boundary value
+     * @param value  initial value for the control
+     * @param step   step size to use for up/down arrow clicks
+     * @param precision number of digits after the decimal point
+     * @param parent parent QWidget
+     * @param name   internal name for this widget
+     * @since 3.1
+     */
+    KDoubleNumInput(double lower, double upper, double value, double step=0.01,
+		    int precision=2, QWidget *parent=0, const char *name=0);
 
     /**
      * destructor
@@ -383,76 +473,123 @@ public:
     virtual ~KDoubleNumInput();
 
     /**
+     * @deprecated (rounds @p value to a multiple of 1/100)
      * Constructor
      *
-     * put it below other KNumInput
+     * puts it visually below other KNumInput
      *
      * @param  below
      * @param  value  initial value for the control
      * @param  parent parent QWidget
      * @param  name   internal name for this widget
      **/
-    KDoubleNumInput(KNumInput* below, double value, QWidget* parent=0, const char* name=0);
+    KDoubleNumInput(KNumInput* below, double value, QWidget* parent=0, const char* name=0) KDE_DEPRECATED;
+
+    /**
+     * Constructor
+     *
+     * the difference here is the "below" parameter. It tells this
+     * instance that it is visually put below some other KNumInput
+     * widget.  Note that these two KNumInput's need not to have the
+     * same parent widget or be in the same layout group.  The effect
+     * is that it'll adjust it's layout in correspondence with the
+     * layout of the other KNumInput's (you can build an arbitrary long
+     * chain).
+     *
+     * @param below  append KDoubleNumInput to the KDoubleNumInput chain
+     * @param lower lower boundary value
+     * @param upper upper boundary value
+     * @param value  initial value for the control
+     * @param step   step size to use for up/down arrow clicks
+     * @param precision number of digits after the decimal point
+     * @param parent parent QWidget
+     * @param name   internal name for this widget
+     * @since 3.1
+     */
+    KDoubleNumInput(KNumInput* below,
+		    double lower, double upper, double value, double step=0.02,
+		    int precision=2, QWidget *parent=0, const char *name=0);
 
     /**
      * @return the current value.
      */
     double value() const;
+
     /**
      * @return the suffix.
-     * @see #setSuffix()
+     * @see setSuffix()
      */
     QString suffix() const;
+
     /**
      * @return the prefix.
-     * @see #setPrefix()
+     * @see setPrefix()
      */
     QString prefix() const;
+
     /**
-     * @return the format.
-     * @see #setFormat()
+     * @return the precision.
+     * @see setPrecision()
      */
-    const char *format() const;
+    int precision() const;
+
     /**
      * @return the string displayed for a special value.
-     * @see #setSpecialValueText()
+     * @see setSpecialValueText()
      */
     QString specialValueText() const { return m_specialvalue; }
 
      /**
-     * @param lower  lower bound on range
-     * @param upper  upper bound on range
-     * @param step   step size for the QSlider
+     * @param min  minimum value
+     * @param max  maximum value
+     * @param step step size for the QSlider
+     * @param slider whether the slider is created or not
      */
-    void setRange(double lower, double upper, double step=1, bool slider=true);
+    void setRange(double min, double max, double step=1, bool slider=true);
+    /**
+     * Sets the minimum value.
+     */
+    void setMinValue(double min);
+    /**
+     * @return the minimum value.
+     */
+    double minValue() const;
+    /**
+     * Sets the maximum value.
+     */
+    void setMaxValue(double max);
+    /**
+     * @return the maximum value.
+     */
+    double maxValue() const;
 
     /**
-     * Sets the format string that should be used to display the double value.
-     * The format string follows the same rules as the printf() function or
-     * @ref QString::sprintf(). You can use this if you want to set the number
-     * of digits to be displayed, etc.
+     * Specifies the number of digits to use.
      */
-    void setFormat(const char* format);
+    void setPrecision(int precision);
+
+    /**
+     * @return the reference point for relativeValue calculation
+     * @since 3.1
+     */
+    double referencePoint() const;
+
+    /**
+     * @return the current value in units of referencePoint.
+     * @since 3.1
+     */
+    double relativeValue() const;
 
     /**
      * Sets the special value text. If set, the spin box will display
      * this text instead of the numeric value whenever the current
-     * value is equal to @ref #minVal(). Typically this is used for indicating
+     * value is equal to minVal(). Typically this is used for indicating
      * that the choice has a special (default) meaning.
      */
     void setSpecialValueText(const QString& text);
 
-    /**
-     * @reimplemented
-     */
-    virtual void setLabel(QString label, int a = AlignLeft | AlignTop);
-    /**
-     * @reimplemented
-     */
+    virtual void setLabel(const QString & label, int a = AlignLeft | AlignTop);
     virtual QSize minimumSizeHint() const;
-    /**
-     * @reimplemented
-     */
     virtual bool eventFilter(QObject*, QEvent*);
 
 public slots:
@@ -461,12 +598,26 @@ public slots:
      */
     void setValue(double);
 
+    /**
+     * Sets the value in units of referencePoint.
+     * @since 3.1
+     */
+    void setRelativeValue(double);
+
+    /**
+     * Sets the reference Point to @p ref. It @p ref == 0, emitting of
+     * relativeValueChanged is blocked and relativeValue
+     * just returns 0.
+     * @since 3.1
+     */
+    void setReferencePoint(double ref);
+
     /**
      * Sets the suffix to be displayed to @p suffix. Use QString::null to disable
      * this feature. Note that the suffix is attached to the value without any
      * spacing. So if you prefer to display a space separator, set suffix
      * to something like " cm".
-     * @see #setSuffix()
+     * @see setSuffix()
      */
     void setSuffix(const QString &suffix);
 
@@ -474,48 +625,59 @@ public slots:
      * Sets the prefix to be displayed to @p prefix. Use QString::null to disable
      * this feature. Note that the prefix is attached to the value without any
      * spacing.
-     * @see #setPrefix()
+     * @see setPrefix()
      */
     void setPrefix(const QString &prefix);
 
 signals:
     /**
-     * Emitted every time the value changes (by calling @ref setValue() or
+     * Emitted every time the value changes (by calling setValue() or
      * by user interaction).
      */
     void valueChanged(double);
+    /**
+     * This is an overloaded member function, provided for
+     * convenience. It essentially behaves like the above function.
+     *
+     * Contains the value in units of referencePoint.
+     * @since 3.1
+     */
+    void relativeValueChanged(double);
 
 private slots:
     void sliderMoved(int);
+    void slotEmitRelativeValueChanged(double);
 
 protected:
-
-    /**
-     * @reimplemented
-     */
     virtual void doLayout();
-    /**
-     * @reimplemented
-     */
     void resizeEvent ( QResizeEvent * );
+
     virtual void resetEditBox();
 
+    // ### no longer used, remove when BIC allowed
     KDoubleLine*   edit;
 
     bool     m_range;
     double   m_lower, m_upper, m_step;
+    // ### end no longer used
 
     QSize    m_sizeEdit;
 
     friend class KDoubleLine;
-
 private:
-    void init(double value);
-    QString  m_units, m_specialvalue, m_prefix, m_suffix;
-    char     *m_format;
+    void init(double value, double lower, double upper,
+	      double step, int precision);
+    double mapSliderToSpin(int) const;
+    void updateLegacyMembers();
+    // ### no longer used, remove when BIC allowed:
+    QString  m_specialvalue, m_prefix, m_suffix;
     double   m_value;
-    int      m_sliderstep; // currently unused
+    short    m_precision;
+    // ### end remove when BIC allowed
 
+protected:
+    virtual void virtual_hook( int id, void* data );
+private:
     class KDoubleNumInputPrivate;
     KDoubleNumInputPrivate *d;
 };
@@ -524,12 +686,13 @@ private:
 /* ------------------------------------------------------------------------ */
 
 /**
- *  An integer inputline with scrollbar and slider.
+ *  A QSpinBox with support for arbitrary base numbers
+ *  (e.g. hexadecimal).
  *
  *  The class provides an easy interface to use other
- *  numeric systems then the decimal.
+ *  numeric systems than the decimal.
  *
- *  @short A spin box widget for non-decimal numbers.
+ *  @short A QSpinBox with support for arbitrary base numbers.
  */
 class KIntSpinBox : public QSpinBox
 {
@@ -567,12 +730,7 @@ public:
     /**
      *  Destructor.
      */
-    virtual ~KIntSpinBox() {};
-
-     /**
-     *  Reimplemented for internal reasons; the API is not affected. 
-     */
-    int value () const;
+    virtual ~KIntSpinBox();
 
     /**
      * Sets the base in which the numbers in the spin box are represented.
@@ -591,22 +749,22 @@ public:
 protected:
 
     /**
-     *  Overloaded to make use of the base given in the constructor.
+     *  Overloaded the method in QSpinBox
+     *  to make use of the base given in the constructor.
      */
     virtual QString mapValueToText(int);
 
     /**
-     *  Overloaded to make use of the base given in the constructor.
+     *  Overloaded the method in QSpinBox
+     *  to make use of the base given in the constructor.
      */
     virtual int mapTextToValue(bool*);
 
-    /**
-     * @reimplemented
-     */
-    virtual void focusInEvent(QFocusEvent*);
-
 private:
     int val_base;
+protected:
+    virtual void virtual_hook( int id, void* data );
+private:
     class KIntSpinBoxPrivate;
     KIntSpinBoxPrivate *d;
 };
@@ -614,5 +772,165 @@ private:
 
 /* --------------------------------------------------------------------------- */
 
+/**
+   This class provides a spin box for fractional numbers.
+
+   \b Parameters \n
+
+   There are a number of interdependent parameters whose relation to
+   each other you need to understand in order to make successful use
+   of the spin box.
+
+   @li precision: The number of decimals after the decimal point.
+   @li maxValue/minValue: upper and lower bound of the valid range
+   @li lineStep: the size of the step that is taken when the user hits
+                 the up or down buttons
+
+   Since we work with fixed-point numbers internally, the maximum
+   precision is a function of the valid range and vice versa. More
+   precisely, the following relations hold:
+   \code
+   max( abs(minValue()), abs(maxValue() ) <= INT_MAX/10^precision
+   maxPrecision = floor( log10( INT_MAX/max(abs(minValue()),abs(maxValue())) ) )
+   \endcode
+
+   Since the value, bounds and step are rounded to the current
+   precision, you may experience that the order of setting above
+   parameters matters. E.g. the following are @em not equivalent (try
+   it!):
+
+   \code
+   // sets precision,
+   // then min/max value (rounded to precision and clipped to obtainable range if needed)
+   // then value and lineStep
+   KDoubleSpinBox * spin = new KDoubleSpinBox( 0, 9.999, 0.001, 4.321, 3, this );
+
+   // sets minValue to 0; maxValue to 10.00(!); value to 4.32(!) and only then
+   // increases the precision - too late, since e.g. value has already been rounded...
+   KDpubleSpinBox * spin = new KDoubleSpinBox( this );
+   spin->setMinValue( 0 );
+   spin->setMaxValue( 9.999 );
+   spin->setValue( 4.321 );
+   spin->setPrecision( 3 );
+   \endcode
+
+   @short A spin box for fractional numbers.
+   @author Marc Mutz <mutz@kde.org>
+   @version $Id: knuminput.h,v 1.56 2003/10/07 22:53:26 mueller Exp $
+   @since 3.1
+**/
+
+class KDoubleSpinBox : public QSpinBox {
+  Q_OBJECT
+  Q_PROPERTY( bool acceptLocalizedNumbers READ acceptLocalizedNumbers WRITE setAcceptLocalizedNumbers )
+  Q_OVERRIDE( double maxValue READ maxValue WRITE setMaxValue )
+  Q_OVERRIDE( double minValue READ minValue WRITE setMinValue )
+  Q_OVERRIDE( double lineStep READ lineStep WRITE setLineStep )
+  Q_OVERRIDE( double value READ value WRITE setValue )
+  Q_PROPERTY( int precision READ precision WRITE setPrecision )
+
+public:
+  /** Constructs a KDoubleSpinBox with parent @p parent and
+      default values for range and value (whatever QRangeControl
+      uses) and precision (2). */
+  KDoubleSpinBox( QWidget * parent=0, const char * name=0 );
+  /** Constructs a KDoubleSpinBox with parent @p parent, range
+      [@p lower,@p upper], lineStep @p step, precision @p
+      precision and initial value @p value. */
+  KDoubleSpinBox( double lower, double upper, double step, double value,
+		  int precision=2, QWidget * parent=0, const char * name=0 );
+
+  virtual ~KDoubleSpinBox();
+
+  /** @return whether the spinbox uses localized numbers */
+  bool acceptLocalizedNumbers() const;
+  /** Sets whether to use and accept localized numbers as returned by
+ KLocale::formatNumber() */
+  virtual void setAcceptLocalizedNumbers( bool accept );
+
+  /** Sets a new range for the spin box values. Note that @p lower, @p
+      upper and @p step are rounded to @p precision decimal points
+      first. */
+  void setRange( double lower, double upper, double step=0.01, int precision=2 );
+
+  /** @return the current number of decimal points displayed. */
+  int precision() const;
+  /** Equivalent to setPrecsion( @p precision, @p false ); Needed
+      since Qt's moc doesn't ignore trailing parameters with default
+      args when searching for a property setter method. */
+  void setPrecision( int precision );
+  /** Sets the number of decimal points to use. Note that there is a
+      tradeoff between the precision used and the available range of
+      values. See the class docs for more.
+      @param precision the new number of decimal points to use
+
+      @param force disables checking of bound violations that can
+             arise if you increase the precision so much that the
+             minimum and maximum values can't be represented
+             anymore. Disabling is useful if you don't want to keep
+             the current min and max values anyway. This is what
+             e.g. setRange() does.
+  **/
+  virtual void setPrecision( int precision, bool force );
+
+  /** @return the current value */
+  double value() const;
+  /** @return the current lower bound */
+  double minValue() const;
+  /** Sets the lower bound of the range to @p value, subject to the
+      contraints that @p value is first rounded to the current
+      precision and then clipped to the maximum representable
+      interval.
+      @see maxValue, minValue, setMaxValue, setRange
+  */
+  void setMinValue( double value );
+  /** @return the current upper bound */
+  double maxValue() const;
+  /** Sets the upper bound of the range to @p value, subject to the
+      contraints that @p value is first rounded to the current
+      precision and then clipped to the maximum representable
+      interval.
+      @see minValue, maxValue, setMinValue, setRange
+  */
+  void setMaxValue( double value );
+
+  /** @return the current step size */
+  double lineStep() const;
+  /** Sets the step size for clicking the up/down buttons to @p step,
+      subject to the constraints that @p step is first rounded to the
+      current precision and then clipped to the meaningful interval
+      [1, @p maxValue - @p minValue]. */
+  void setLineStep( double step );
+
+  /** Overridden to ignore any setValidator() calls. */
+  void setValidator( const QValidator * );
+
+signals:
+  /** Emitted whenever QSpinBox::valueChanged( int ) is emitted. */
+  void valueChanged( double value );
+
+public slots:
+  /** Sets the current value to @p value, cubject to the constraints
+      that @p value is frist rounded to the current precision and then
+      clipped to the interval [@p minvalue(),@p maxValue()]. */
+  virtual void setValue( double value );
+
+protected:
+  virtual QString mapValueToText(int);
+  virtual int mapTextToValue(bool*);
+
+protected slots:
+  void slotValueChanged( int value );
+
+protected:
+ virtual void virtual_hook( int id, void* data );
+private:
+  typedef QSpinBox base;
+  void updateValidator();
+  int maxPrecision() const;
+
+  class Private;
+  Private * d;
+};
 
 #endif // K_NUMINPUT_H
