@@ -25,9 +25,10 @@
 #define KTOOLBAR_H
 
 #include <qtoolbar.h>
+#include <qmainwindow.h>
 #include <qcombobox.h>
 #include <qmap.h>
-#include <qlist.h>
+#include <qptrlist.h>
 #include <kglobal.h>
 #include <qguardedptr.h>
 #include <qframe.h>
@@ -67,6 +68,7 @@ public slots:
    void setOrientation( Orientation );
 protected:
     void styleChange( QStyle& );
+    void drawContents( QPainter* );
 private:
     Orientation orient;
     bool line;
@@ -74,26 +76,27 @@ private:
 
 
  /**
+  * @short Floatable toolbar with auto resize.
+  *
   * A KDE-style toolbar.
   *
   * KToolBar can be dragged around in and between different docks.
   *
   * A KToolBar can contain all sorts of widgets.
   *
-  * KToolBar can be used as a standalone widget, but @ref KMainWindow
+  * KToolBar can be used as a standalone widget, but KMainWindow
   * provides easy factories and management of one or more toolbars.
   * Once you have a KToolBar object, you can insert items into it with the
-  * insert... methods, or remove them with the @ref removeItem() method. This
+  * insert... methods, or remove them with the removeItem() method. This
   * can be done at any time; the toolbar will be automatically updated.
   * There are also many methods to set per-child properties like alignment
-  * and toggle behaviour.
+  * and toggle behavior.
   *
   * KToolBar uses a global config group to load toolbar settings on
   * construction. It will reread this config group on a
-  * @ref KApplication::appearanceChanged() signal.
+  * KApplication::appearanceChanged() signal.
   *
-  * @short Floatable toolbar with auto resize.
-  * @version $Id: ktoolbar.h,v 1.131 2001/03/31 12:06:47 pfeiffer Exp $
+  * @version $Id: ktoolbar.h,v 1.176 2003/11/05 11:43:22 tokoe Exp $
   * @author Reginald Stadlbauer <reggie@kde.org>, Stephan Kulow <coolo@kde.org>, Sven Radej <radej@kde.org>.
   */
 
@@ -112,87 +115,90 @@ public:
     /**
      * The state of the status bar.
      * @deprecated
-     **/
+     */
     enum BarStatus{Toggle, Show, Hide};
     /**
    * Possible bar positions.
-   **/
+   */
     enum BarPosition{ Unmanaged, Floating, Top, Bottom, Right, Left, Flat};
 
   /**
-   * Constructor.
+   * Normal constructor.
    * This constructor is used by the XML-GUI. If you use it, you need
    * to call QMainWindow::addToolBar to specify the position of the toolbar.
    * So it's simpler to use the other constructor.
    *
    * The toolbar will read in various global config settings for
    * things like icon size and text position, etc.  However, some of
-   * the settings will be honored only if @ref #_honor_mode is set to
+   * the settings will be honored only if @p honorStyle is set to
    * true.  All other toolbars will be IconOnly and use Medium icons.
    *
    * @param parent      The standard toolbar parent (usually a
-   *                    @ref KMainWindow)
+   * KMainWindow)
    * @param name        The standard internal name
-   * @param honor_style If true, then global settings for IconSize and IconText will be honored
+   * @param honorStyle  If true, then global settings for IconSize and IconText will be honored
    * @param readConfig  whether to apply the configuration (global and application-specific)
    */
-    KToolBar( QWidget *parent, const char *name = 0, bool honor_style = FALSE, bool readConfig = TRUE );
+    KToolBar( QWidget *parent, const char *name = 0, bool honorStyle = false, bool readConfig = true );
 
   /**
    * Constructor for non-XML-GUI applications.
    *
    * The toolbar will read in various global config settings for
    * things like icon size and text position, etc.  However, some of
-   * the settings will be honored only if @ref #_honor_mode is set to
+   * the settings will be honored only if @p honorStyle is set to
    * true.  All other toolbars will be IconOnly and use Medium icons.
    *
    * @param parentWindow The window that should be the parent of this toolbar
    * @param dock        The position of the toolbar. Usually QMainWindow::Top.
    * @param newLine     If true, start a new line in the dock for this toolbar.
    * @param name        The standard internal name
-   * @param honor_style If true, then global settings for IconSize and IconText will be honored
+   * @param honorStyle  If true, then global settings for IconSize and IconText will be honored
    * @param readConfig  whether to apply the configuration (global and application-specific)
    */
     KToolBar( QMainWindow *parentWindow, QMainWindow::ToolBarDock dock /*= QMainWindow::Top*/, bool newLine = false,
-              const char *name = 0, bool honor_style = FALSE, bool readConfig = TRUE );
+              const char *name = 0, bool honorStyle = false, bool readConfig = true );
 
   /**
    * Constructor for non-XML-GUI applications.
    *
    * The toolbar will read in various global config settings for
    * things like icon size and text position, etc.  However, some of
-   * the settings will be honored only if @ref #_honor_mode is set to
+   * the settings will be honored only if @p honorStyle is set to
    * true.  All other toolbars will be IconOnly and use Medium icons.
    *
    * @param parentWindow The window that should be the parent of this toolbar
    * @param dock        Another widget than the mainwindow to dock toolbar to.
    * @param newLine     If true, start a new line in the dock for this toolbar.
    * @param name        The standard internal name
-   * @param honor_style If true, then global settings for IconSize and IconText will be honored
+   * @param honorStyle  If true, then global settings for IconSize and IconText will be honored
    * @param readConfig  whether to apply the configuration (global and application-specific)
    */
     KToolBar( QMainWindow *parentWindow, QWidget *dock, bool newLine = false,
-              const char *name = 0, bool honor_style = FALSE, bool readConfig = TRUE );
+              const char *name = 0, bool honorStyle = false, bool readConfig = true );
 
+  /**
+   * Destructor
+   */
     virtual ~KToolBar();
 
   /**
-   * Insert a button (a @ref KToolbarButton) with a pixmap.  The
+   * Insert a button (a KToolBarButton) with a pixmap.  The
    * pixmap is loaded by the button itself based on the global icon
    * settings.
    *
    * You should connect to one or more signals in KToolBar:
-   * @ref clicked() , @ref pressed() , @ref released() , or
-   * @ref highlighted()  and if the button is a toggle button
-   * (@ref setToggle() ) @ref toggled() .  Those signals have @p id
+   * clicked() , pressed() , released() , or
+   * highlighted()  and if the button is a toggle button
+   * ( setToggle() ) toggled() .  Those signals have @p id
    * of a button that caused the signal.  If you want to bind a popup
-   * to button, see @ref setButton().
+   * to button, see setButton().
    *
    * @param icon    The name of the icon to use as the active pixmap
    * @param id      The id of this button
    * @param enabled Enable or disable the button at startup
    * @param text    The tooltip or toolbar text (depending on state)
-   * @param index The position of the button. (-1 = at end).
+   * @param index   The position of the button. (-1 = at end).
    *
    * @return The item index.
    */
@@ -200,16 +206,17 @@ public:
 		     const QString& text = QString::null, int index=-1,
 		     KInstance *_instance = KGlobal::instance());
 
-    /**
+   /**
    * This is the same as above, but with specified signals and
    * slots to which this button will be connected.
    *
-   * You can add more signals with @ref addConnection().
+   * You can add more signals with addConnection().
    *
    * @param icon     The name of the icon to use as the active pixmap
    * @param id       The id of this button
    * @param signal   The signal to connect to
    * @param receiver The slot's parent
+   * @param slot     The slot to receive the signal specified in that argument.
    * @param enabled  Enable or disable the button at startup
    * @param text     The tooltip or toolbar text (depending on state)
    * @param index    The position of the button. (-1 = at end).
@@ -221,8 +228,8 @@ public:
 		     bool enabled = true, const QString& text = QString::null,
 		     int index=-1, KInstance *_instance = KGlobal::instance() );
 
-    /**
-   * Insert a button (a @ref KToolbarButton) with the specified
+  /**
+   * Inserts a button (a KToolBarButton) with the specified
    * pixmap.  This pixmap will be used as the "active" one and the
    * disabled and default ones will be autogenerated.
    *
@@ -231,33 +238,34 @@ public:
    * itself.  Specifying the icon name is much more flexible.
    *
    * You should connect to one or more signals in KToolBar:
-   * @ref clicked() , @ref pressed() , @ref released() , or
-   * @ref highlighted()  and if the button is a toggle button
-   * (@ref setToggle() ) @ref toggled() .  Those signals have @p id
+   * clicked() , pressed() , released() , or
+   * highlighted()  and if the button is a toggle button
+   * ( setToggle() ) toggled() .  Those signals have @p id
    * of a button that caused the signal.  If you want to bind a popup
-   * to button, see @ref setButton().
+   * to button, see setButton().
    *
    * @param pixmap  The active pixmap
    * @param id      The id of this button
    * @param enabled Enable or disable the button at startup
    * @param text    The tooltip or toolbar text (depending on state)
-   * @param index The position of the button. (-1 = at end).
+   * @param index   The position of the button. (-1 = at end).
    *
    * @return The item index.
    */
     int insertButton(const QPixmap& pixmap, int id, bool enabled = true,
 		     const QString& text = QString::null, int index=-1 );
 
-    /**
+  /**
    * This is the same as above, but with specified signals and
    * slots to which this button will be connected.
    *
-   * You can add more signals with @ref addConnection().
+   * You can add more signals with addConnection().
    *
-   * @param icon     The name of the icon to use as the active pixmap
+   * @param pixmap   The name of the icon to use as the active pixmap
    * @param id       The id of this button
    * @param signal   The signal to connect to
    * @param receiver The slot's parent
+   * @param slot     The slot to receive the signal specified in that argument.
    * @param enabled  Enable or disable the button at startup
    * @param text     The tooltip or toolbar text (depending on state)
    * @param index    The position of the button. (-1 = at end).
@@ -269,25 +277,40 @@ public:
 		     bool enabled = true, const QString& text = QString::null,
 		     int index=-1 );
 
-    /**
-   * Insert a button with popupmenu.
+  /**
+   * Inserts a button with popupmenu.
    *
    * Button will have small
    * triangle. You have to connect to popup's signals. The
-   * signals @ref KButton::pressed(), @ref KButton::released(),
-   * @ref KButton::clicked() or @ref KButton::doubleClicked() are @p not
+   * signals KButton::pressed(), KButton::released(),
+   * KButton::clicked() or KButton::doubleClicked() are @p not
    * emmited by
-   * this button (see @ref setDelayedPopup() for that).
-   * You can add custom popups which inherit @ref QPopupMenu to get popups
+   * this button (see setDelayedPopup() for that).
+   * You can add custom popups which inherit QPopupMenu to get popups
+   * with tables, drawings etc. Just don't fiddle with events there.
+   */
+    int insertButton(const QString& icon, int id, QPopupMenu *popup,
+		     bool enabled, const QString&_text, int index=-1);
+
+  /**
+   * Inserts a button with popupmenu.
+   *
+   * Button will have small
+   * triangle. You have to connect to popup's signals. The
+   * signals KButton::pressed(), KButton::released(),
+   * KButton::clicked() or KButton::doubleClicked() are @p not
+   * emmited by
+   * this button (see setDelayedPopup() for that).
+   * You can add custom popups which inherit QPopupMenu to get popups
    * with tables, drawings etc. Just don't fiddle with events there.
    */
     int insertButton(const QPixmap& pixmap, int id, QPopupMenu *popup,
 		     bool enabled, const QString&_text, int index=-1);
 
-    /**
+  /**
    * Inserts a KLineEdit. You have to specify signals and slots to
    * which KLineEdit will be connected. KLineEdit has all slots QLineEdit
-   * has, plus signals @ref KLineEdit::completion and @ref KLineEdit::rotation
+   * has, plus signals KLineEdit::completion and KLineEdit::textRotation
    * KLineEdit can be set to autoresize itself to full free width
    * in toolbar, that is to last right aligned item. For that,
    * toolbar must be set to full width (which it is by default).
@@ -303,29 +326,8 @@ public:
 		     const QString& toolTipText = QString::null,
 		     int size = 70, int index =-1);
 
-    /**
-   * Insert @ref KComboBox with list.
-   *
-   * Can be writable, but cannot contain
-   * pixmaps. By default inserting policy is AtBottom, i.e. typed items
-   * are placed at the bottom of the list. Can be autosized. If the size
-   * argument is specified as -1, the width of the combobox is automatically
-   * computed.
-   *
-   * @see setFullWidth()
-   * @see setItemAutoSized()
-   * @see KComboBox
-   * @return Item index.
-   */
-    int insertCombo (QStrList *list, int id, bool writable,
-		     const char *signal, const QObject *receiver,
-		     const char *slot, bool enabled=true,
-		     const QString& tooltiptext=QString::null,
-		     int size=70, int index=-1,
-		     QComboBox::Policy policy = QComboBox::AtBottom);
-
-    /**
-   * Insert @ref KComboBox with list.
+  /**
+   * Inserts a KComboBox with list.
    *
    * Can be writable, but cannot contain
    * pixmaps. By default inserting policy is AtBottom, i.e. typed items
@@ -345,76 +347,48 @@ public:
 		     int size=70, int index=-1,
 		     QComboBox::Policy policy = QComboBox::AtBottom);
 
-    /**
-   * Insert @ref KComboBox with text.
+  /**
+   * Insert a KComboBox with text.
    *
-   *  The rest is the same as above.
+   * The rest is the same as above.
    * @see setItemAutoSized()
    *
    * @see KComboBox
    * @return Item index.
    */
     int insertCombo (const QString& text, int id, bool writable,
-		     const char *signal, QObject *recevier,
+		     const char *signal, QObject *receiver,
 		     const char *slot, bool enabled=true,
 		     const QString& tooltiptext=QString::null,
 		     int size=70, int index=-1,
 		     QComboBox::Policy policy = QComboBox::AtBottom);
-    /**
-   * Insert separator.
-   */
-    int insertSeparator(int index=-1);
 
     /**
-     * Insert a separator into the toolbar with the given id.
-     * Returns the separator's index
+     * Inserts a separator into the toolbar with the given @p id.
+     * @return the separator's index
      */
-    int insertSeparator( int index, int id ); // ### KDE 3.0: BCI: merge
+    int insertSeparator( int index = -1, int id = -1 );
 
     /**
-   * Insert line separator.
-   */
-    int insertLineSeparator(int index=-1);
-
-    /**
-     * Insert a line separator into the toolbar with the given id.
-     * Returns the separator's index
+     * Inserts a line separator into the toolbar with the given @p id.
+     * @return the separator's index
      */
-    int insertLineSeparator( int index, int id ); // ### KDE 3.0: BCI: merge
+    int insertLineSeparator( int index = -1, int id = -1 );
 
-    /**
-   * Insert a user-defined widget.  The widget @p must have this
+  /**
+   * Inserts a user-defined widget.  The widget @p must have this
    * toolbar as its parent.
    *
    * Widget must have a QWidget for base class.  Widget can be
    * autosized to full width. If you forget about it, you can get a
-   * pointer to this widget with @ref getWidget().
+   * pointer to this widget with getWidget().
    * @see setItemAutoSized()
    * @return Item index.
    */
     int insertWidget(int id, int width, QWidget *_widget, int index=-1);
 
   /**
-   * Insert an animated widget.  A @ref KAnimWidget will be created
-   * internally using the icon list you provide.
-   * This will emit a signal (clicked()) whenever the
-   * animation widget is clicked.
-   *
-   * @see animatedWidget()
-   *
-   * @param id       The id for this toolbar item
-   * @param receiver The parent of your slot
-   * @param slot     The slot to receive the clicked() signal
-   * @param icons    A list of all icon names to use for the animation
-   * @param index    The item index
-   *
-   * @return The item index
-   */
-    int insertAnimatedWidget(int id, QObject *receiver, const char *slot,
-			     const QStringList& icons, int index = -1);
-
-  /**
-   * Insert an animated widget.  A @ref KAnimWidget will be created
+   * Inserts an animated widget.  A KAnimWidget will be created
    * internally using the icon name you provide.
    * This will emit a signal (clicked()) whenever the
    * animation widget is clicked.
@@ -433,53 +407,51 @@ public:
 			     const QString& icons, int index = -1);
 
     /**
-   * This will return a pointer to the given animated widget, if it
-   * exists.
+   * This will return a pointer to the animated widget with the
+   * given @p id, if it exists.
    *
    * @see insertAnimatedWidget
-   *
    * @param id The id for the widget you want to get a pointer to
-   *
    * @return A pointer to the current animated widget or 0L
    */
     KAnimWidget *animatedWidget( int id );
 
-    /**
+  /**
    * Adds connections to items.
    *
    * It is important that you
    * know the @p id of particular item. Nothing happens if you forget @p id.
    */
     void addConnection (int id, const char *signal,
-			const QObject *receiver, const char *slot);
-    /**
-   * Enable/disable item.
+                        const QObject *receiver, const char *slot);
+  /**
+   * Enables/disables item.
    */
     void setItemEnabled( int id, bool enabled );
 
-    /**
-   * Set button pixmap.
-   *
-   * Can be used while button is visible.
-   */
-    void setButtonPixmap( int id, const QPixmap& _pixmap );
-
-    /**
-   * Set button icon.
+  /**
+   * Sets the icon for a button.
    *
    * Can be used while button is visible.
    */
     void setButtonIcon( int id, const QString& _icon );
 
-    /**
-   * Set button icons from a QIconSet.
+  /**
+   * Sets button pixmap.
+   *
+   * Can be used while button is visible.
+   */
+    void setButtonPixmap( int id, const QPixmap& _pixmap );
+
+  /**
+   * Sets a button icon from a QIconSet.
    *
    * Can be used while button is visible.
    */
     void setButtonIconSet( int id, const QIconSet& iconset );
 
-    /**
-   * Set delayed popup for a button.
+  /**
+   * Sets a delayed popup for a button.
    *
    * Delayed popup is what you see in
    * Netscape Navigator's Previous and Next buttons: If you click them you
@@ -487,25 +459,27 @@ public:
    * or forth. If you press them long enough, you get a history-menu.
    * This is exactly what we do here.
    *
-   * You will insert normal a button with connection (or use signals from
+   * You will insert a normal button with connection (or use signals from
    * toolbar):
-   * <pre>
-   * bar->insertButton(pixmap, id, const SIGNAL(clicked ()), this,
+   * \code
+   * bar->insertButton(icon, id, SIGNAL(clicked ()), this,
    *                   SLOT (slotClick()), true, "click or wait for popup");
-   * </pre> And then add a delayed popup:
-   * <pre>
-   * bar->setDelayedPopup (id, historyPopup); </pre>
+   * \endcode
+   * And then add a delayed popup:
+   * \code
+   * bar->setDelayedPopup (id, historyPopup);
+   * \endcode
    *
    * Don't add delayed popups to buttons which have normal popups.
    *
-   * You may add popups which are derived from @ref QPopupMenu. You may
+   * You may add popups which are derived from QPopupMenu. You may
    * add popups that are already in the menu bar or are submenus of
    * other popups.
    */
     void setDelayedPopup (int id , QPopupMenu *_popup, bool toggle = false);
 
-    /**
-   * Make a button an autorepeat button.
+  /**
+   * Turns a button into an autorepeat button.
    *
    * Toggle buttons, buttons with menus, or
    * buttons with delayed menus cannot be made into autorepeat buttons.
@@ -514,141 +488,138 @@ public:
    * When the user presses this button, you will receive the signal clicked(),
    * and if the button is still pressed after some time,
    * you will receive more clicked() signals separated by regular
-   * intervals.  Since this uses @ref QButton::setAutoRepeat() ,
+   * intervals.  Since this uses QButton::setAutoRepeat() ,
    * I can't quantify 'some'.
    */
     void setAutoRepeat (int id, bool flag=true);
 
 
-    /**
-   * Make button a toggle button if @p flag is true.
+  /**
+   * Turns button into a toggle button if @p flag is true.
    */
     void setToggle (int id, bool flag = true);
 
-    /**
-   * Toggle a togglebutton.
+  /**
+   * Toggles a togglebutton.
    *
-   * If the button is a toggle button  (see @ref setToggle())
+   * If the button is a toggle button  (see setToggle())
    * the button state will be toggled. This will also cause the toolbar to
-   * emit the signal @ref KButton::toggled() with parameter @p id. You must connect to
-   * this signal, or use @ref addConnection() to connect directly to the
-   * button signal @ref KButton::toggled().
+   * emit the signal KButton::toggled() with parameter @p id. You must connect to
+   * this signal, or use addConnection() to connect directly to the
+   * button signal KButton::toggled().
    */
     void toggleButton (int id);
 
-    /**
-   * See a toggle button state.
+  /**
+   * Sets a toggle button state.
    *
-   * If the button is a toggle button (see @ref setToggle())
+   * If the button is a toggle button (see setToggle())
    * this will set its state flag. This will also emit the signal
-   * @ref KButton::toggled().
+   * KButton::toggled().
    *
    * @see setToggle()
    */
     void setButton (int id, bool flag);
 
-    /**
-   * @return @p true if button is on, @p false if button is off or if the
-   * button is not a toggle button.
+  /**
+   * Returns @p true if button is on and is a toggle button
+   * @return @p true if button is on and is a toggle button
    * @see setToggle()
    */
-    bool isButtonOn (int id);
+    bool isButtonOn (int id) const;
 
-    /**
-   * Set text in line editor.
+  /**
+   * Sets the text of a line editor.
    *
    * Cursor is set at end of text.
    */
     void setLinedText (int id, const QString& text);
 
-    /**
-   * Retreive line editor text.
-   *
-   * If you want to store this text, you have to deep-copy it somewhere.
+  /**
+   * Returns the line editor text.
+   * @return the line editor text.
    */
-    QString getLinedText (int id);
+    QString getLinedText (int id) const;
 
-    /**
-   * Insert text in combobox @p id at position @p index.
+  /**
+   * Inserts @p text in combobox @p id at position @p index.
    */
     void insertComboItem (int id, const QString& text, int index);
 
     /**
-   * Insert list in combobox @p id at position @p index.
-   */
-    void insertComboList (int id, QStrList *list, int index);
-
-    /**
-   * Insert list in combobox @p id at position @p index.
+   * Inserts @p list in combobox @p id at position @p index.
    */
     void insertComboList (int id, const QStringList &list, int index);
 
-    /**
-   * Remove item @p index from combobox @p id.
+  /**
+   * Removes item @p index from combobox @p id.
    */
     void removeComboItem (int id, int index);
 
-    /**
-   * Set item @p index to be current item in combobox @p id.
+  /**
+   * Sets item @p index to be current item in combobox @p id.
    */
     void setCurrentComboItem (int id, int index);
 
-    /**
-   * Change item @p index in combobox @p id to text.
+  /**
+   * Changes item @p index in combobox @p id to text.
    *
    * @p index = -1 refers current item (one displayed in the button).
    */
     void changeComboItem  (int id, const QString& text, int index=-1);
 
-    /**
-   * Clear combobox @p id.
+  /**
+   * Clears the combobox @p id.
    *
    * Does not delete it or hide it.
    */
     void clearCombo (int id);
 
-    /**
-   * Return text of item @p index from combobox @p id.
+  /**
+   * Returns text of item @p index from combobox @p id.
+   * @return text of item @p index from combobox @p id.
    *
    * @p index = -1 refers to current item.
    */
 
-    QString getComboItem (int id, int index=-1);
+    QString getComboItem (int id, int index=-1) const;
 
     /**
-   * Retrieve a pointer to the combobox.
+   * Returns a pointer to the combobox with @p id.
+   * @return a pointer to the combobox with @p id.
    *
    * Example:
-   * <pre>
+   * \code
    * KComboBox *combo = toolbar->getCombo(combo_id);
-   * </pre>
+   * \endcode
    * That way you can get access to other public methods
-   * that @ref KComboBox provides.
+   * that KComboBox provides.
    */
     KComboBox * getCombo(int id);
 
     /**
-   * Retrieve a pointer to KToolBarLined.
+   * Returns a pointer to KToolBarLined with @p id.
+   * @return a pointer to KToolBarLined with @p id.
    *
    * Example:
-   * <pre>
-   * KLineEdit * lined = toolbar->getKTollBarLined(lined_id);
-   * </pre>
+   * \code
+   * KLineEdit * lined = toolbar->getKToolBarLined(lined_id);
+   * \endcode
    * That way you can get access to other public methods
-   * that @ref KLineEdit provides. @ref KLineEdit is the same thing
-   * as @ref QLineEdit plus completion signals.
+   * that KLineEdit provides. KLineEdit is the same thing
+   * as QLineEdit plus completion signals.
    */
     KLineEdit * getLined (int id);
 
     /**
-   * Retrieve a pointer to KToolBarButton.
+   * Returns a pointer to KToolBarButton.
    *
    * Example:
-   * <pre>
+   * \code
    * KToolBarButton * button = toolbar->getButton(button_id);
-   * </pre>
+   * \endcode
    * That way you can get access to other public methods
-   * that @ref KToolBarButton provides.
+   * that KToolBarButton provides.
    *
    *  Using this method is not recommended.
    */
@@ -663,16 +634,17 @@ public:
     void alignItemRight (int id, bool right = true);
 
     /**
-   * Retrieve a pointer to an inserted widget.
+   * Returns a pointer to the widget corresponding to @p id.
+   * @return a pointer to the widget corresponding to @p id.
    *
    * Wrong ids are not tested.
    * You can do with this whatever you want,
    * except change its height (hardcoded). If you change its width
-   * you will probbably have to call toolbar->@ref updateRects (true)
+   * you will probably have to call QToolBar::updateRects(true)
    * @see QWidget
    * @see updateRects()
    */
-    QWidget *getWidget (int id);
+    QWidget *getWidget (int id); // ### KDE4: make this const!
 
     /**
    * Set item autosized.
@@ -704,23 +676,43 @@ public:
     void removeItem (int id);
 
     /**
-   * Hide item.
+   * Remove item @p id.
+   *
+   * Item is deleted when toolbar is redrawn.
+   */
+    void removeItemDelayed (int id);
+
+    /**
+   * Hide item @p id.
    */
     void hideItem (int id);
 
     /**
-   * Show item.
+   * Show item @p id.
    */
     void showItem (int id);
 
+    /**
+   * Returns the index of the item @p id.
+   * @return the index of the item @p id.
+   * @since 3.2
+   */
+    int itemIndex (int id); // ### KDE4: make this const!
+
+    /**
+   * Returns the id of the item at the given index.
+   * @since 3.2
+   */
+    int idAt(int index); // ### KDE4: make this const!
+
     /**
    * Set toolbar to full parent size (default).
    *
-   *  In full size mode the bar
-   * extends over the parent's full width or height. If the mode is disabled
-   * the toolbar tries to take as much space as it needs without wrapping, but
-   * it does not exceed the parent box. You can force a certain width or
-   * height with @ref setMaxWidth() or @ref setMaxHeight().
+   * In full size mode the bar extends over the parent's full width or height.
+   * If the mode is disabled the toolbar tries to take as much space as it
+   * needs without wrapping, but it does not exceed the parent box. You can
+   * force a certain width or height with setMaxWidth() or
+   * setMaxHeight().
    *
    * If you want to use right-aligned items or auto-sized items you must use
    * full size mode.
@@ -728,15 +720,16 @@ public:
     void setFullSize(bool flag = true);
 
     /**
-   * @return @p true if the full-size mode is enabled. Otherwise
-   * it returns @false.
+   * Returns the full-size mode enabled flag.
+   * @return @p true if the full-size mode is enabled.
    */
     bool fullSize() const;
 
     /**
    * Enable or disable moving of toolbar.
+   * @deprecated use setMovingEnabled(bool) instead.
    */
-    void enableMoving(bool flag = true);
+    void enableMoving(bool flag = true) KDE_DEPRECATED;
 
     /**
    * Set position of toolbar.
@@ -745,46 +738,49 @@ public:
     void setBarPos (BarPosition bpos);
 
     /**
-   * Retrieve position of toolbar.
+   * Returns the toolbar position.
+   * @return position of toolbar.
    */
     BarPosition barPos() const;
 
   /**
-   * @deprecated
    * Show, hide, or toggle toolbar.
    *
    * This method is provided for compatibility only,
    * please use show() and/or hide() instead.
    * @see BarStatus
-   */
-    bool enable(BarStatus stat);
-
-  /**
    * @deprecated
-   * Use setMaximumHeight() instead.
    */
-    void setMaxHeight (int h);  // Set max height for vertical toolbars
+    bool enable(BarStatus stat) KDE_DEPRECATED;
 
   /**
+   * Use setMaximumHeight() instead.
+   * @deprecated
+   */
+    void setMaxHeight (int h) KDE_DEPRECATED;  // Set max height for vertical toolbars
+
+  /**
+   * Returns the value set with setMaxHeight().
    * @deprecated
    * Use maximumHeight() instead.
-   * Retrieve the value that was set with @ref setMaxHeight().
+   * @return the value set with setMaxHeight().
    */
-    int maxHeight();
+    int maxHeight() KDE_DEPRECATED;
 
   /**
-   * @deprecated
    * Use setMaximumWidth() instead.
    * Set maximal width of horizontal (top or bottom) toolbar.
+   * @deprecated
    */
-    void setMaxWidth (int dw);
+    void setMaxWidth (int dw) KDE_DEPRECATED;
 
   /**
-   * @deprecated
+   * Returns the value set with setMaxWidth().
    * Use maximumWidth() instead.
-   * Retrieve the value that was set with @ref setMaxWidth().
+   * @return the value set with setMaxWidth().
+   * @deprecated
    */
-    int maxWidth();
+    int maxWidth() KDE_DEPRECATED;
 
   /**
    * Set title for toolbar when it floats.
@@ -795,10 +791,10 @@ public:
     void setTitle (const QString& _title);
 
   /**
+   * Use setMovingEnabled(bool) instead.
    * @deprecated
-   * Use enableMoving() instead.
    */
-    void enableFloating (bool arrrrrrgh);
+    void enableFloating (bool flag) KDE_DEPRECATED;
 
   /**
    * Set the kind of painting for buttons.
@@ -815,7 +811,7 @@ public:
    // Note: don't merge with the next one, it breaks Qt properties
 
   /**
-   * Similar to @ref setIconText(IconText it) but allows you to
+   * Similar to setIconText(IconText it) but allows you to
    * disable or enable updating.  If @p update is false, then the
    * buttons will not be updated.  This is useful only if you know
    * that you will be forcing an update later.
@@ -823,7 +819,8 @@ public:
     void setIconText(IconText it, bool update);
 
   /**
-   * @return The current text style for buttons.
+   * Returns the current text style for buttons.
+   * @return the current text style for buttons.
    */
     IconText iconText() const;
 
@@ -841,7 +838,7 @@ public:
     // Note: don't merge with the next one, it breaks Qt properties
 
   /**
-   * Same as @ref setIconText(int size) but allows you
+   * Same as setIconText(int size) but allows you
    * to disable the toolbar update.
    *
    * @param size   The size to use
@@ -851,7 +848,8 @@ public:
     void setIconSize(int size, bool update);
 
   /**
-   * @return The current icon size for buttons.
+   * Returns the current icon size for buttons.
+   * @return the current icon size for buttons.
    */
     int iconSize() const;
 
@@ -863,9 +861,8 @@ public:
     void setEnableContextMenu(bool enable = true);
 
   /**
-   * Returns whether or not the context menu is disabled
-   *
-   * @return The context menu state
+   * Returns the context menu enabled flag
+   * @return true if the context menu is disabled
    */
     bool contextMenuEnabled() const;
 
@@ -882,16 +879,18 @@ public:
     void setFlat (bool flag);
 
   /**
-   * @return the number of items in the toolbar
+   * Returns the total number of items in the toolbar
+   * @return the total number of items in the toolbar
    */
-    int count();
+    int count() const;
 
   /**
    * Instruct the toolbar to save it's current state to either the app
    * config file or to the XML-GUI resource file (whichever has
    * precedence).
    */
-    void saveState();
+    void saveState(); // BIC: remove for KDE4? This doesn't appear to be used internally,
+                      // and apps use saveMainWindowSettings in KMainWindow anyway.
 
   /**
    * Save the toolbar settings to group @p configGroup in @p config.
@@ -904,18 +903,19 @@ public:
    */
     void applySettings(KConfig *config, const QString &configGroup);
 
-  /**
+  /*
    * Tell the toolbar what XML-GUI resource file it should use to save
    * it's state.  The state of the toolbar (position, size, etc) is
-   * saved in KConfig files if the application does not use XML-GUI..
+   * saved in KConfig files if the application does not use XML-GUI
    * but if the app does, then it's saved the XML file.  This function
    * allows this to happen.
    *
    * @param xmlfile The XML-GUI resource file to write to
    * @param xml     The DOM document for the XML-GUI building
+   * @internal
    */
   //  void setXML(const QString& xmlfile, const QDomDocument& xml);
-  /* @internal */
+
     void setXMLGUIClient( KXMLGUIClient *client );
 
   /**
@@ -925,6 +925,7 @@ public:
     void setText( const QString & txt );
 
   /**
+   * Returns the toolbar's text.
    * @return the toolbar's text.
    */
     QString text() const;
@@ -932,11 +933,14 @@ public:
     void setStretchableWidget( QWidget *w );
     QSizePolicy sizePolicy() const;
     bool highlight() const;
+    QSize sizeHint() const;
+    QSize minimumSizeHint() const;
+    QSize minimumSize() const;
 
     void hide();
     void show();
 
-    void updateRects( bool = FALSE ) {}
+    void updateRects( bool = false ) {}
 
     void loadState( const QDomElement &e );
     void saveState( QDomElement &e );
@@ -944,11 +948,7 @@ public:
     /**
      * @internal
      */
-    void positionYourself();
-    /**
-     * @internal
-     */
-    void positionYourself( bool force ); // BCI: to be merged
+    void positionYourself( bool force = false);
 
 signals:
     /**
@@ -960,10 +960,10 @@ signals:
      * Emitted when button @p id is double-clicked.
      *
      * Note: you will always
-     * recive two @ref clicked() , @ref pressed() and @ref released() signals.
+     * recive two clicked() , pressed() and released() signals.
      * There is no way to avoid it - at least no easy way.
      * If you need to resolve this all you can do is set up timers
-     * which wait for @ref QApplication::doubleClickInterval() to expire.
+     * which wait for QApplication::doubleClickInterval() to expire.
      * If in that time you don't get this signal, you may belive that
      * button was only clicked and not double-clicked.
      * And please note that butons with popup menus do not emit this signal,
@@ -985,15 +985,15 @@ signals:
      * Emitted when a toggle button changes state.
      *
      * Emitted also if you change state
-     * with @ref setButton() or @ref toggleButton()
+     * with setButton() or toggleButton()
      * If you make a button normal again, with
-     * @ref setToggle (false), this signal won't
+     * setToggle(false), this signal won't
      * be emitted.
      */
     void toggled(int);
 
     /**
-     * This signal is emitted when item id gets highlighted/unhighlighted
+     * This signal is emitted when item @p id gets highlighted/unhighlighted
      * (i.e when mouse enters/exits).
      *
      * Note that this signal is emitted from
@@ -1005,7 +1005,7 @@ signals:
     void highlighted(int id, bool isHighlighted);
 
     /**
-     * This signal is emitted when item id gets highlighted/unhighlighted
+     * This signal is emitted when item @p id gets highlighted/unhighlighted
      * (i.e when mouse enters/exits).
      *
      * Note that this signal is emitted from
@@ -1018,40 +1018,56 @@ signals:
      * Emitted when toolbar changes position, or when
      * an item is removed from toolbar.
      *
-     * If you subclass @ref KMainWindow and reimplement
-     * @ref KMainWindow::resizeEvent() be sure to connect to
+     * If you subclass KMainWindow and reimplement
+     * KMainWindow::resizeEvent() be sure to connect to
      * this signal. Note: You can connect this signal to a slot that
      * doesn't take parameter.
      */
     void moved( BarPosition );
 
     /**
-     * @internal
-     * This signal is emited when toolbar detects changing of
+     * This signal is emitted when toolbar detects changing of
      * following parameters:
      * highlighting, button-size, button-mode. This signal is
      * internal, aimed to buttons.
+     * @internal
      */
     void modechange ();
 
     /**
+     * This signal is emitted when the toolbar is getting deleted,
+     * and before ~KToolbar finishes (so it's still time to remove
+     * widgets from the toolbar).
+     * Used by KWidgetAction.
+     * @since 3.2
+     */
+    void toolbarDestroyed();
+
+public:
+    /**
+     * Returns the global setting for "Highlight buttons under mouse"
      * @return global setting for "Highlight buttons under mouse"
      */
     static bool highlightSetting();
 
     /**
+     * Returns the global setting for "Toolbars transparent when moving"
      * @return global setting for "Toolbars transparent when moving"
      */
     static bool transparentSetting();
 
     /**
+     * Returns the global setting for "Icon Text"
      * @return global setting for "Icon Text"
      */
     static IconText iconTextSetting();
 
+public slots:
+    virtual void setIconText( const QString &txt )
+    { QToolBar::setIconText( txt ); }
+
 protected:
     void mousePressEvent( QMouseEvent * );
-    void paintEvent(QPaintEvent*);
     void childEvent( QChildEvent *e );
     void showEvent( QShowEvent *e );
     void resizeEvent( QResizeEvent *e );
@@ -1067,6 +1083,7 @@ private slots:
     void slotRepaint();
     void toolBarPosChanged( QToolBar *tb );
     void slotContextAboutToShow();
+    void slotContextAboutToHide();
     void widgetDestroyed();
 
 private:
@@ -1074,18 +1091,21 @@ private:
     void doConnections( KToolBarButton *button );
     void insertWidgetInternal( QWidget *w, int &index, int id );
     void removeWidgetInternal( QWidget *w );
-    void getAttributes( QString &position, QString &icontext, QString &index, QString &offset, QString &newLine );
+    void getAttributes( QString &position, QString &icontext, int &index );
+    int dockWindowIndex();
     KPopupMenu *contextMenu();
 
     QMap<QWidget*, int > widget2id;
     typedef QMap<int, QWidget* > Id2WidgetMap;
     Id2WidgetMap id2widget;
-    KToolBarPrivate *d;
     KPopupMenu *context;
-    QList<QWidget> widgets, inserted /*to be removed*/;
+    QPtrList<QWidget> widgets;
     QTimer *layoutTimer;
     QGuardedPtr<QWidget> stretchableWidget, rightAligned;
-
+protected:
+    virtual void virtual_hook( int id, void* data );
+private:
+    KToolBarPrivate *d;
 };
 
 #endif
